MODULE hdfeos5_module

!
!HEADER
!
!Purpose:
!   To define named constants
!

IMPLICIT NONE

include 'hdfeos5.inc'	! include standard hdf-eos5 Fortran header

! define functions

integer, external :: he5_swopen, he5_swclose,he5_swidtype
integer, external :: he5_swattach, he5_swdetach,he5_swlattrinfo
integer, external :: he5_swrdattr, he5_ehrdglatt
integer, external :: he5_swfldinfo, he5_swinqswath
integer, external :: he5_swrdfld, he5_ehinqglatts
integer, external :: he5_swcreate,he5_swdefdim,he5_swinqattrs
integer, external :: he5_swdefdfld,he5_swdefgfld,he5_swrdlattr
integer, external :: he5_ehwrglatt,he5_swwrgattr, he5_swsetfill
integer, external :: he5_swinqdims,he5_swdiminfo,he5_swwrlattr
integer, external :: he5_swwrfld,he5_swinqdflds,he5_swinqgflds


END MODULE hdfeos5_module
