module metadata_module

  implicit none
  !
  integer, parameter :: metadata_field_length   = 50
  integer, parameter :: metadata_value_length   = 100

  type Tmetadata
    character(len=metadata_field_length)   :: fieldname
    character(len=metadata_value_length)   :: value
  end type Tmetadata
 
 
end module metadata_module
